use crate::fan::Fan;

use rppal::gpio::{Gpio, OutputPin};

pub struct Hysteresis {
    pub max_temperature: f64,
    pub hysteresis: f64,
    pin: OutputPin,
}

impl Hysteresis {
    pub fn new(max_temperature: f64, hysteresis: f64, bcm_pin: u8) -> Hysteresis {
        let gpio = Gpio::new().unwrap();
        let mut pin = gpio.get(bcm_pin).unwrap().into_output();

        pin.set_low();

        Hysteresis {
            max_temperature,
            hysteresis,
            pin,
        }
    }
}

impl Fan for Hysteresis {
    fn update_temperature(&mut self, temperature: f64) {
        if temperature > self.max_temperature {
            self.pin.set_high();
        } else if temperature < self.max_temperature - self.hysteresis {
            self.pin.set_low();
        }
    }

    fn percentage(&self) -> f64 {
        if self.pin.is_set_high() {
            1_f64
        } else {
            0_f64
        }
    }
}
