use crate::fan::Fan;

use rppal::pwm::{Channel, Polarity, Pwm as Pwm_rpi};

pub struct Pwm {
    pub max_temperature: f64,
    pub min_temperature: f64,
    pwm: Pwm_rpi,
}

impl Pwm {
    pub fn new(max_temperature: f64, min_temperature: f64, bcm_pin: Channel) -> Pwm {
        let pwm =
            Pwm_rpi::with_frequency(bcm_pin, 0.1_f64, 0_f64, Polarity::Normal, false).unwrap();

        Pwm {
            max_temperature,
            min_temperature,
            pwm,
        }
    }
}

impl Fan for Pwm {
    fn update_temperature(&mut self, temperature: f64) {
        let percentage =
            (temperature - self.min_temperature) / (self.max_temperature - self.min_temperature);
        let percentage = if percentage > 1_f64 {
            1_f64
        } else {
            percentage
        };

        if temperature > self.min_temperature {
            self.pwm.enable().unwrap();
            self.pwm.set_duty_cycle(percentage).unwrap();
        } else {
            self.pwm.disable().unwrap();
        }
    }

    fn percentage(&self) -> f64 {
        self.pwm.duty_cycle().unwrap()
    }
}
