mod fan;
mod temperatures;

use fan::hysteresis::Hysteresis;
use fan::pwm::Pwm;
use fan::Fan;

use rppal::pwm::Channel;
use std::error::Error;
use std::thread;
use std::time::Duration;

fn main() -> Result<(), Box<dyn Error>> {
    let mut temps = temperatures::Temperatures::new()?;

    let mut fan: Box<dyn Fan> = if true {
        Box::new(Hysteresis::new(52.0, 5.0, 18))
    } else {
        Box::new(Pwm::new(52.0, 42.0, Channel::Pwm0))
    };

    loop {
        temps.update();

        match temps.max() {
            Some(t) => {
                fan.update_temperature(t);

                println!(
                    "CPU: {:.1}°C, fan is at {:.1}%",
                    t,
                    fan.percentage() * 100_f64
                );
            }
            None => {
                eprintln!("Recuring problem while getting the temperature");
            }
        }

        thread::sleep(Duration::from_secs(5));
    }
}
